@@ -10,6 +10,7 @@
 
 
 (require 'geiser-company)
+(require 'geiser-compile)
 (require 'geiser-doc)
 (require 'geiser-autodoc)
 (require 'geiser-edit)
@@ -183,17 +184,26 @@ module command as a string")
 (defsubst geiser-repl--repl-name (impl)
   (format "%s REPL" (geiser-impl--impl-str impl)))
 
+(defsubst geiser-repl--buffer-name (impl)
+  (format "* %s *" (geiser-repl--repl-name impl)))
+
+(defun geiser-repl--switch-to-buffer (buffer)
+  (unless (eq buffer (current-buffer))
+    (let ((pop-up-windows geiser-repl-window-allow-split))
+      (if geiser-repl-use-other-window
+          (switch-to-buffer-other-window buffer)
+        (switch-to-buffer buffer)))))
+
 (defun geiser-repl--to-repl-buffer (impl)
   (unless (and (eq major-mode 'geiser-repl-mode)
+               (eq geiser-impl--implementation impl)
                (not (get-buffer-process (current-buffer))))
     (let* ((old (geiser-repl--repl/impl impl geiser-repl--closed-repls))
            (old (and (buffer-live-p old)
                      (not (get-buffer-process old))
                      old)))
-      (pop-to-buffer
-       (or old
-           (generate-new-buffer (format "* %s *"
-                                        (geiser-repl--repl-name impl)))))
+      (geiser-repl--switch-to-buffer
+       (or old (generate-new-buffer (geiser-repl--buffer-name impl))))
       (unless old
         (geiser-repl-mode)
         (geiser-impl--set-buffer-implementation impl)))))
@@ -243,6 +253,9 @@ module command as a string")
                                         (geiser-repl--host)
                                         (geiser-repl--port)))))
 
+(defun geiser-repl--update-debugging (txt)
+  (geiser-con--connection-update-debugging geiser-repl--connection txt))
+
 (defun geiser-repl--start-repl (impl address)
   (message "Starting Geiser REPL for %s ..." impl)
   (geiser-repl--to-repl-buffer impl)
@@ -266,6 +279,11 @@ module command as a string")
                                        deb-prompt-rx))
     (geiser-repl--startup impl address)
     (geiser-repl--autodoc-mode 1)
+    (geiser-company--setup geiser-repl-company-p)
+    (add-hook 'comint-output-filter-functions
+              'geiser-repl--update-debugging
+              nil
+              t)
     (message "%s up and running!" (geiser-repl--repl-name impl))))
 
 (defun geiser-repl--start-scheme (impl address prompt)
@@ -417,7 +435,10 @@ module command as a string")
     (let ((beginning-of-defun-function nil))
       (beginning-of-defun))))
 
-(defun geiser-repl--module-function (&optional ignore) :f)
+(defun geiser-repl--module-function (&optional module)
+  (if (and module geiser-eval--get-impl-module)
+      (funcall geiser-eval--get-impl-module module)
+      :f))
 
 (defun geiser-repl--doc-module ()
   (interactive)
@@ -478,7 +499,7 @@ module command as a string")
 there's no symbol at point). Otherwise, go to next error in the REPL
 buffer."
   (interactive "p")
-  (if (> (point) (geiser-repl--last-prompt-end))
+  (if (>= (point) (geiser-repl--last-prompt-end))
       (completion-at-point)
     (compilation-next-error n)))
 
@@ -503,7 +524,6 @@ buffer."
   (geiser-completion--setup t)
   (setq geiser-smart-tab-mode-string "")
   (geiser-smart-tab-mode t)
-  (geiser-company--setup geiser-repl-company-p)
   ;; enabling compilation-shell-minor-mode without the annoying highlighter
   (compilation-setup t))
 
@@ -527,6 +547,7 @@ buffer."
   --
   ("Switch to module..." "\C-c\C-m" switch-to-geiser-module)
   ("Import module..." "\C-c\C-i" geiser-repl-import-module)
+  ("Add to load path..." "\C-c\C-r" geiser-add-to-load-path)
   --
   ("Previous matching input" "\M-p" comint-previous-matching-input-from-input
    "Previous input matching current")
@@ -586,13 +607,12 @@ If no REPL is running, execute `run-geiser' to start a fresh one."
                      ((and (not ask)
                            (not in-repl)
                            impl
-                           (geiser-repl--repl/impl impl)))))
-         (pop-up-windows geiser-repl-window-allow-split))
+                           (geiser-repl--repl/impl impl))))))
     (cond ((or in-live-repl
                (and (eq (current-buffer) repl) (not (eq repl buffer))))
            (when (buffer-live-p geiser-repl--last-scm-buffer)
-             (pop-to-buffer geiser-repl--last-scm-buffer)))
-          (repl (pop-to-buffer repl))
+             (geiser-repl--switch-to-buffer geiser-repl--last-scm-buffer)))
+          (repl (geiser-repl--switch-to-buffer repl))
           ((geiser-repl--remote-p) (geiser-connect impl))
           (t (run-geiser impl)))
     (when (and buffer (eq major-mode 'geiser-repl-mode))
